# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from devtools_testutils.aio import recorded_by_proxy_async
from testpreparer import MultiPartPreparer
from testpreparer_async import MultiPartClientTestBaseAsync


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestMultiPartFormDataOperationsAsync(MultiPartClientTestBaseAsync):
    @MultiPartPreparer()
    @recorded_by_proxy_async
    async def test_form_data_basic(self, multipart_endpoint):
        client = self.create_async_client(endpoint=multipart_endpoint)
        response = await client.form_data.basic(
            body={"id": "str", "profileImage": "filetype"},
        )

        # please add some check logic here by yourself
        # ...

    @MultiPartPreparer()
    @recorded_by_proxy_async
    async def test_form_data_with_wire_name(self, multipart_endpoint):
        client = self.create_async_client(endpoint=multipart_endpoint)
        response = await client.form_data.with_wire_name(
            body={"id": "str", "profileImage": "filetype"},
        )

        # please add some check logic here by yourself
        # ...

    @MultiPartPreparer()
    @recorded_by_proxy_async
    async def test_form_data_optional_parts(self, multipart_endpoint):
        client = self.create_async_client(endpoint=multipart_endpoint)
        response = await client.form_data.optional_parts(
            body={"id": "str", "profileImage": "filetype"},
        )

        # please add some check logic here by yourself
        # ...

    @MultiPartPreparer()
    @recorded_by_proxy_async
    async def test_form_data_file_array_and_basic(self, multipart_endpoint):
        client = self.create_async_client(endpoint=multipart_endpoint)
        response = await client.form_data.file_array_and_basic(
            body={"address": {"city": "str"}, "id": "str", "pictures": ["filetype"], "profileImage": "filetype"},
        )

        # please add some check logic here by yourself
        # ...

    @MultiPartPreparer()
    @recorded_by_proxy_async
    async def test_form_data_json_part(self, multipart_endpoint):
        client = self.create_async_client(endpoint=multipart_endpoint)
        response = await client.form_data.json_part(
            body={"address": {"city": "str"}, "profileImage": "filetype"},
        )

        # please add some check logic here by yourself
        # ...

    @MultiPartPreparer()
    @recorded_by_proxy_async
    async def test_form_data_binary_array_parts(self, multipart_endpoint):
        client = self.create_async_client(endpoint=multipart_endpoint)
        response = await client.form_data.binary_array_parts(
            body={"id": "str", "pictures": ["filetype"]},
        )

        # please add some check logic here by yourself
        # ...

    @MultiPartPreparer()
    @recorded_by_proxy_async
    async def test_form_data_multi_binary_parts(self, multipart_endpoint):
        client = self.create_async_client(endpoint=multipart_endpoint)
        response = await client.form_data.multi_binary_parts(
            body={"profileImage": "filetype", "picture": "filetype"},
        )

        # please add some check logic here by yourself
        # ...

    @MultiPartPreparer()
    @recorded_by_proxy_async
    async def test_form_data_check_file_name_and_content_type(self, multipart_endpoint):
        client = self.create_async_client(endpoint=multipart_endpoint)
        response = await client.form_data.check_file_name_and_content_type(
            body={"id": "str", "profileImage": "filetype"},
        )

        # please add some check logic here by yourself
        # ...

    @MultiPartPreparer()
    @recorded_by_proxy_async
    async def test_form_data_anonymous_model(self, multipart_endpoint):
        client = self.create_async_client(endpoint=multipart_endpoint)
        response = await client.form_data.anonymous_model(
            body={"profileImage": "filetype"},
        )

        # please add some check logic here by yourself
        # ...

    @MultiPartPreparer()
    @recorded_by_proxy_async
    async def test_form_data_http_parts_json_array_and_file_array(self, multipart_endpoint):
        client = self.create_async_client(endpoint=multipart_endpoint)
        response = await client.form_data.http_parts.json_array_and_file_array(
            body={
                "address": {"city": "str"},
                "id": "str",
                "pictures": ["filetype"],
                "previousAddresses": [{"city": "str"}],
                "profileImage": "filetype",
            },
        )

        # please add some check logic here by yourself
        # ...

    @MultiPartPreparer()
    @recorded_by_proxy_async
    async def test_form_data_file_upload_file_specific_content_type(self, multipart_endpoint):
        client = self.create_async_client(endpoint=multipart_endpoint)
        response = await client.form_data.file.upload_file_specific_content_type(
            body={"file": "filetype"},
        )

        # please add some check logic here by yourself
        # ...

    @MultiPartPreparer()
    @recorded_by_proxy_async
    async def test_form_data_file_upload_file_required_filename(self, multipart_endpoint):
        client = self.create_async_client(endpoint=multipart_endpoint)
        response = await client.form_data.file.upload_file_required_filename(
            body={"file": "filetype"},
        )

        # please add some check logic here by yourself
        # ...

    @MultiPartPreparer()
    @recorded_by_proxy_async
    async def test_form_data_file_upload_file_array(self, multipart_endpoint):
        client = self.create_async_client(endpoint=multipart_endpoint)
        response = await client.form_data.file.upload_file_array(
            body={"files": ["filetype"]},
        )

        # please add some check logic here by yourself
        # ...

    @MultiPartPreparer()
    @recorded_by_proxy_async
    async def test_form_data_http_parts_content_type_image_jpeg_content_type(self, multipart_endpoint):
        client = self.create_async_client(endpoint=multipart_endpoint)
        response = await client.form_data.http_parts.content_type.image_jpeg_content_type(
            body={"profileImage": "filetype"},
        )

        # please add some check logic here by yourself
        # ...

    @MultiPartPreparer()
    @recorded_by_proxy_async
    async def test_form_data_http_parts_content_type_required_content_type(self, multipart_endpoint):
        client = self.create_async_client(endpoint=multipart_endpoint)
        response = await client.form_data.http_parts.content_type.required_content_type(
            body={"profileImage": "filetype"},
        )

        # please add some check logic here by yourself
        # ...

    @MultiPartPreparer()
    @recorded_by_proxy_async
    async def test_form_data_http_parts_content_type_optional_content_type(self, multipart_endpoint):
        client = self.create_async_client(endpoint=multipart_endpoint)
        response = await client.form_data.http_parts.content_type.optional_content_type(
            body={"profileImage": "filetype"},
        )

        # please add some check logic here by yourself
        # ...

    @MultiPartPreparer()
    @recorded_by_proxy_async
    async def test_form_data_http_parts_non_string_float(self, multipart_endpoint):
        client = self.create_async_client(endpoint=multipart_endpoint)
        response = await client.form_data.http_parts.non_string.float(
            body={"temperature": 0.0},
        )

        # please add some check logic here by yourself
        # ...



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from enum import Enum
from azure.core import CaseInsensitiveEnumMeta


class ClientType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of ClientType."""

    DEFAULT = "default"
    """DEFAULT."""
    MULTI_CLIENT = "multi-client"
    """MULTI_CLIENT."""
    RENAMED_OPERATION = "renamed-operation"
    """RENAMED_OPERATION."""
    TWO_OPERATION_GROUP = "two-operation-group"
    """TWO_OPERATION_GROUP."""
    CLIENT_OPERATION_GROUP = "client-operation-group"
    """CLIENT_OPERATION_GROUP."""



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
# pylint: disable=useless-super-delegation

from typing import Any, Mapping, Optional, overload

from .._utils.model_base import Model as _Model, rest_field


class SafeintAsStringProperty(_Model):
    """SafeintAsStringProperty.

    :ivar value: Required.
    :vartype value: int
    """

    value: int = rest_field(visibility=["read", "create", "update", "delete", "query"], format="str")
    """Required."""

    @overload
    def __init__(
        self,
        *,
        value: int,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class Uint32AsStringProperty(_Model):
    """Uint32AsStringProperty.

    :ivar value:
    :vartype value: int
    """

    value: Optional[int] = rest_field(visibility=["read", "create", "update", "delete", "query"], format="str")

    @overload
    def __init__(
        self,
        *,
        value: Optional[int] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class Uint8AsStringProperty(_Model):
    """Uint8AsStringProperty.

    :ivar value: Required.
    :vartype value: int
    """

    value: int = rest_field(visibility=["read", "create", "update", "delete", "query"], format="str")
    """Required."""

    @overload
    def __init__(
        self,
        *,
        value: int,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)



import argparse
import os
import sys
import subprocess

from typing import Optional, List
from subprocess import CalledProcessError

from ci_tools.functions import install_into_venv
from ci_tools.variables import in_ci, discover_repo_root, set_envvar_defaults
from ci_tools.environment_exclusions import is_check_enabled
from ci_tools.logging import logger

from .Check import Check

BLACK_VERSION = "24.4.0"
REPO_ROOT = discover_repo_root()


class black(Check):
    def __init__(self) -> None:
        super().__init__()

    def register(
        self, subparsers: "argparse._SubParsersAction", parent_parsers: Optional[List[argparse.ArgumentParser]] = None
    ) -> None:
        """Register the `black` check. The black check installs black and runs black against the target package."""
        parents = parent_parsers or []
        p = subparsers.add_parser("black", parents=parents, help="Run the code formatter black check")
        p.set_defaults(func=self.run)

    def run(self, args: argparse.Namespace) -> int:
        """Run the black check command."""
        logger.info("Running black check...")

        set_envvar_defaults()

        targeted = self.get_targeted_directories(args)

        results: List[int] = []

        for parsed in targeted:
            if os.getcwd() != parsed.folder:
                os.chdir(parsed.folder)
            package_dir = parsed.folder
            package_name = parsed.name

            executable, staging_directory = self.get_executable(
                args.isolate,
                args.command,
                sys.executable,
                package_dir,
                python_version=getattr(args, "python_version", None),
            )
            logger.info(f"Processing {package_name} for black check")

            self.install_dev_reqs(executable, args, package_dir)

            logger.info(f"Running black against {package_name}")

            check_only = in_ci() != 0

            if check_only:
                if not is_check_enabled(package_dir, "black", default=False):
                    logger.info(f"Package {package_name} opts-out of black check.")
                    continue

            result = self.format_directory(executable, package_dir, check_only=check_only)
            if result is None:
                results.append(1)
                continue

            stderr_text = result.stderr.decode("utf-8") if result.stderr else ""
            stdout_text = result.stdout.decode("utf-8") if result.stdout else ""

            has_issues = result.returncode != 0 if check_only else "reformatted" in stderr_text

            if has_issues:
                if check_only and result.returncode > 1:
                    logger.error(f"Black failed for {package_name} (exit code {result.returncode}).")
                    if stderr_text.strip():
                        logger.error(f"Black stderr:\n{stderr_text}")
                    results.append(result.returncode)
                elif check_only:
                    logger.info(
                        f"The package {package_name} has black formatting issues. "
                        f"Run `azpysdk black .` locally from the package root to reformat."
                    )
                    if stdout_text.strip():
                        logger.info(f"Black diff output:\n{stdout_text}")
                    if stderr_text.strip():
                        logger.info(f"Black summary:\n{stderr_text}")
                    results.append(1)
                else:
                    logger.info(f"The package {package_name} was reformatted.")
                    if stdout_text.strip():
                        logger.info(f"Black diff output:\n{stdout_text}")
                    if stderr_text.strip():
                        logger.info(f"Black summary:\n{stderr_text}")
            else:
                logger.info(f"The package {package_name} is properly formatted, no files changed.")

        return max(results) if results else 0

    @staticmethod
    def format_directory(
        executable: str, target_dir: str, check_only: bool = False
    ) -> Optional[subprocess.CompletedProcess]:
        """Run black on *target_dir* using the repo-wide config.

        Installs the pinned black version into the environment of *executable*,
        then formats all Python files under *target_dir*.

        When *check_only* is True, runs with ``--check --diff`` to report
        issues without modifying files.
        """
        try:
            install_into_venv(executable, [f"black=={BLACK_VERSION}"], target_dir)
        except CalledProcessError as e:
            logger.error(f"Failed to install black, skipping formatting: {e}")
            return None

        config_file_location = os.path.join(REPO_ROOT, "eng/black-pyproject.toml")
        cmd = [executable, "-m", "black", f"--config={config_file_location}"]
        if check_only:
            cmd.append("--check")
            cmd.append("--diff")
        cmd.append(target_dir)

        try:
            return subprocess.run(
                cmd,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                check=not check_only,
            )
        except subprocess.CalledProcessError as e:
            logger.error(f"Black formatting failed for {target_dir}: {e}")
            if e.stderr:
                logger.error(e.stderr.decode("utf-8"))
            return None



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from copy import deepcopy
from typing import Any, Awaitable
from typing_extensions import Self

from azure.core import AsyncPipelineClient
from azure.core.pipeline import policies
from azure.core.rest import AsyncHttpResponse, HttpRequest

from .._utils.serialization import Deserializer, Serializer
from ._configuration import RenamedFromClientConfiguration
from .operations import NewInterfaceOperations, _RenamedFromClientOperationsMixin


class RenamedFromClient(_RenamedFromClientOperationsMixin):  # pylint: disable=client-accepts-api-version-keyword
    """Test for the ``@renamedFrom`` decorator.

    :ivar new_interface: NewInterfaceOperations operations
    :vartype new_interface: versioning.renamedfrom.aio.operations.NewInterfaceOperations
    :param endpoint: Need to be set as '`http://localhost:3000 <http://localhost:3000>`_' in
     client. Required.
    :type endpoint: str
    :keyword version: Need to be set as 'v1' or 'v2' in client. Known values are "v2" and None.
     Default value is None. If not set, the operation's default API version will be used. Note that
     overriding this default value may result in unsupported behavior.
    :paramtype version: str or ~versioning.renamedfrom.models.Versions
    """

    def __init__(  # pylint: disable=missing-client-constructor-parameter-credential
        self, endpoint: str, **kwargs: Any
    ) -> None:
        _endpoint = "{endpoint}/versioning/renamed-from/api-version:{version}"
        self._config = RenamedFromClientConfiguration(endpoint=endpoint, **kwargs)

        _policies = kwargs.pop("policies", None)
        if _policies is None:
            _policies = [
                policies.RequestIdPolicy(**kwargs),
                self._config.headers_policy,
                self._config.user_agent_policy,
                self._config.proxy_policy,
                policies.ContentDecodePolicy(**kwargs),
                self._config.redirect_policy,
                self._config.retry_policy,
                self._config.authentication_policy,
                self._config.custom_hook_policy,
                self._config.logging_policy,
                policies.DistributedTracingPolicy(**kwargs),
                policies.SensitiveHeaderCleanupPolicy(**kwargs) if self._config.redirect_policy else None,
                self._config.http_logging_policy,
            ]
        self._client: AsyncPipelineClient = AsyncPipelineClient(base_url=_endpoint, policies=_policies, **kwargs)

        self._serialize = Serializer()
        self._deserialize = Deserializer()
        self._serialize.client_side_validation = False
        self.new_interface = NewInterfaceOperations(self._client, self._config, self._serialize, self._deserialize)

    def send_request(
        self, request: HttpRequest, *, stream: bool = False, **kwargs: Any
    ) -> Awaitable[AsyncHttpResponse]:
        """Runs the network request through the client's chained policies.

        >>> from azure.core.rest import HttpRequest
        >>> request = HttpRequest("GET", "https://www.example.org/")
        <HttpRequest [GET], url: 'https://www.example.org/'>
        >>> response = await client.send_request(request)
        <AsyncHttpResponse: 200 OK>

        For more information on this code flow, see https://aka.ms/azsdk/dpcodegen/python/send_request

        :param request: The network request you want to make. Required.
        :type request: ~azure.core.rest.HttpRequest
        :keyword bool stream: Whether the response payload will be streamed. Defaults to False.
        :return: The response of your network call. Does not do error handling on your response.
        :rtype: ~azure.core.rest.AsyncHttpResponse
        """

        request_copy = deepcopy(request)
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
            "version": self._serialize.url("self._config.version", self._config.version, "str"),
        }

        request_copy.url = self._client.format_url(request_copy.url, **path_format_arguments)
        return self._client.send_request(request_copy, stream=stream, **kwargs)  # type: ignore

    async def close(self) -> None:
        await self._client.close()

    async def __aenter__(self) -> Self:
        await self._client.__aenter__()
        return self

    async def __aexit__(self, *exc_details: Any) -> None:
        await self._client.__aexit__(*exc_details)



# coding=utf-8
# pylint: disable=useless-super-delegation

from typing import Any, Mapping, overload

from ..._utils.model_base import Model as _Model, rest_field

List = list


class DictMethods(_Model):
    """DictMethods.

    :ivar keys_property: Required.
    :vartype keys_property: str
    :ivar items_property: Required.
    :vartype items_property: str
    :ivar values_property: Required.
    :vartype values_property: str
    :ivar popitem_property: Required.
    :vartype popitem_property: str
    :ivar clear_property: Required.
    :vartype clear_property: str
    :ivar update_property: Required.
    :vartype update_property: str
    :ivar setdefault_property: Required.
    :vartype setdefault_property: str
    :ivar pop_property: Required.
    :vartype pop_property: str
    :ivar get_property: Required.
    :vartype get_property: str
    :ivar copy_property: Required.
    :vartype copy_property: str
    """

    keys_property: str = rest_field(
        name="keys", visibility=["read", "create", "update", "delete", "query"], original_tsp_name="keys"
    )
    """Required."""
    items_property: str = rest_field(
        name="items", visibility=["read", "create", "update", "delete", "query"], original_tsp_name="items"
    )
    """Required."""
    values_property: str = rest_field(
        name="values", visibility=["read", "create", "update", "delete", "query"], original_tsp_name="values"
    )
    """Required."""
    popitem_property: str = rest_field(
        name="popitem", visibility=["read", "create", "update", "delete", "query"], original_tsp_name="popitem"
    )
    """Required."""
    clear_property: str = rest_field(
        name="clear", visibility=["read", "create", "update", "delete", "query"], original_tsp_name="clear"
    )
    """Required."""
    update_property: str = rest_field(
        name="update", visibility=["read", "create", "update", "delete", "query"], original_tsp_name="update"
    )
    """Required."""
    setdefault_property: str = rest_field(
        name="setdefault", visibility=["read", "create", "update", "delete", "query"], original_tsp_name="setdefault"
    )
    """Required."""
    pop_property: str = rest_field(
        name="pop", visibility=["read", "create", "update", "delete", "query"], original_tsp_name="pop"
    )
    """Required."""
    get_property: str = rest_field(
        name="get", visibility=["read", "create", "update", "delete", "query"], original_tsp_name="get"
    )
    """Required."""
    copy_property: str = rest_field(
        name="copy", visibility=["read", "create", "update", "delete", "query"], original_tsp_name="copy"
    )
    """Required."""

    @overload
    def __init__(
        self,
        *,
        keys_property: str,
        items_property: str,
        values_property: str,
        popitem_property: str,
        clear_property: str,
        update_property: str,
        setdefault_property: str,
        pop_property: str,
        get_property: str,
        copy_property: str,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class ModelWithList(_Model):
    """ModelWithList.

    :ivar list: Required.
    :vartype list: str
    """

    list: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Required."""

    @overload
    def __init__(
        self,
        *,
        list: str,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class SameAsModel(_Model):
    """SameAsModel.

    :ivar same_as_model: Required.
    :vartype same_as_model: str
    """

    same_as_model: str = rest_field(name="SameAsModel", visibility=["read", "create", "update", "delete", "query"])
    """Required."""

    @overload
    def __init__(
        self,
        *,
        same_as_model: str,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from azure.identity import DefaultAzureCredential

from azure.mgmt.authorization import AuthorizationManagementClient

"""
# PREREQUISITES
    pip install azure-identity
    pip install azure-mgmt-authorization
# USAGE
    python deny_assignments_create_for_subscription.py

    Before run the sample, please set the values of the client ID, tenant ID and client secret
    of the AAD application as environment variables: AZURE_CLIENT_ID, AZURE_TENANT_ID,
    AZURE_CLIENT_SECRET. For more info about how to get the value, please see:
    https://docs.microsoft.com/azure/active-directory/develop/howto-create-service-principal-portal
"""


def main():
    client = AuthorizationManagementClient(
        credential=DefaultAzureCredential(),
        subscription_id="SUBSCRIPTION_ID",
    )

    response = client.deny_assignments.create_or_update(
        scope="subscriptions/a925f2f7-5c63-4b7b-8799-25a5f97bc3b2",
        deny_assignment_id="64b75d79-7a26-4341-944e-4f1a19f0e6ca",
        parameters={
            "properties": {
                "denyAssignmentEffect": "enforced",
                "denyAssignmentName": "Deny delete on critical resources",
                "description": "Prevent all users from deleting critical resources in the subscription.",
                "doNotApplyToChildScopes": False,
                "excludePrincipals": [{"id": "ce2ce14e-85d7-4629-bdbc-454d0519d987", "type": "ServicePrincipal"}],
                "permissions": [{"actions": ["*/delete"], "dataActions": [], "notActions": [], "notDataActions": []}],
                "principals": [{"id": "00000000-0000-0000-0000-000000000000", "type": "SystemDefined"}],
            }
        },
    )
    print(response)


# x-ms-original-file: 2024-07-01-preview/DenyAssignments_CreateForSubscription.json
if __name__ == "__main__":
    main()



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from devtools_testutils import recorded_by_proxy
from testpreparer import HierarchyBuildingClientTestBase, HierarchyBuildingPreparer


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestHierarchyBuildingAnimalOperationsOperations(HierarchyBuildingClientTestBase):
    @HierarchyBuildingPreparer()
    @recorded_by_proxy
    def test_animal_operations_update_pet_as_animal(self, hierarchybuilding_endpoint):
        client = self.create_client(endpoint=hierarchybuilding_endpoint)
        response = client.animal_operations.update_pet_as_animal(
            animal={"breed": "str", "kind": "dog", "name": "str", "trained": bool},
        )

        # please add some check logic here by yourself
        # ...

    @HierarchyBuildingPreparer()
    @recorded_by_proxy
    def test_animal_operations_update_dog_as_animal(self, hierarchybuilding_endpoint):
        client = self.create_client(endpoint=hierarchybuilding_endpoint)
        response = client.animal_operations.update_dog_as_animal(
            animal={"breed": "str", "kind": "dog", "name": "str", "trained": bool},
        )

        # please add some check logic here by yourself
        # ...



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from azure.identity import DefaultAzureCredential

from azure.mgmt.billing import BillingManagementClient

"""
# PREREQUISITES
    pip install azure-identity
    pip install azure-mgmt-billing
# USAGE
    python billing_profiles_create_or_update.py

    Before run the sample, please set the values of the client ID, tenant ID and client secret
    of the AAD application as environment variables: AZURE_CLIENT_ID, AZURE_TENANT_ID,
    AZURE_CLIENT_SECRET. For more info about how to get the value, please see:
    https://docs.microsoft.com/azure/active-directory/develop/howto-create-service-principal-portal
"""


def main():
    client = BillingManagementClient(
        credential=DefaultAzureCredential(),
        subscription_id="SUBSCRIPTION_ID",
    )

    response = client.billing_profiles.begin_create_or_update(
        billing_account_name="00000000-0000-0000-0000-000000000000:00000000-0000-0000-0000-000000000000_2019-05-31",
        billing_profile_name="xxxx-xxxx-xxx-xxx",
        parameters={
            "properties": {
                "billTo": {
                    "addressLine1": "Test Address1",
                    "addressLine2": "Test Address2",
                    "addressLine3": "Test Address3",
                    "city": "City",
                    "companyName": "Contoso",
                    "country": "US",
                    "email": "abc@contoso.com",
                    "firstName": "Test",
                    "isValidAddress": True,
                    "lastName": "User",
                    "phoneNumber": "000-000-0000",
                    "postalCode": "00000",
                    "region": "WA",
                },
                "displayName": "Billing Profile 1",
                "enabledAzurePlans": [{"skuId": "0001"}, {"skuId": "0002"}],
                "invoiceEmailOptIn": True,
                "poNumber": "ABC12345",
                "shipTo": {
                    "addressLine1": "Test Address1",
                    "addressLine2": "Test Address2",
                    "addressLine3": "Test Address3",
                    "city": "City",
                    "companyName": "Contoso",
                    "country": "US",
                    "email": "abc@contoso.com",
                    "firstName": "Test",
                    "isValidAddress": True,
                    "lastName": "User",
                    "phoneNumber": "000-000-0000",
                    "postalCode": "00000",
                    "region": "WA",
                },
            }
        },
    ).result()
    print(response)


# x-ms-original-file: specification/billing/resource-manager/Microsoft.Billing/stable/2024-04-01/examples/billingProfilesCreateOrUpdate.json
if __name__ == "__main__":
    main()



import importlib
import inspect
import json
import logging
import glob
import os
import pkgutil
from pathlib import Path
import subprocess
import sys
import types
import tempfile
import re
from typing import Dict, Any, Optional, List, ForwardRef

# Because I'm subprocessing myself, I need to do weird thing as import.
try:
    # If I'm started as a module __main__
    from .venvtools import create_venv_with_package
except (ModuleNotFoundError, ImportError) as e:
    # If I'm started by my main directly
    from venvtools import create_venv_with_package


_LOGGER = logging.getLogger(__name__)


def parse_input(input_parameter):
    """From a syntax like package_name#submodule, build a package name
    and complete module name.
    """
    split_package_name = input_parameter.split("#")
    package_name = split_package_name[0]
    module_name = package_name.replace("-", ".")
    if len(split_package_name) >= 2:
        module_name = ".".join([module_name, split_package_name[1]])
    return package_name, module_name


def create_empty_report():
    return {"client": {}, "models": {"enums": {}, "exceptions": {}, "models": {}}, "operations": {}}


def is_model(model_cls: object, is_new_model: bool) -> bool:
    return hasattr(model_cls, "_is_model" if is_new_model else "_attribute_map")


def create_report(module_name: str) -> Dict[str, Any]:
    module_to_generate = importlib.import_module(module_name)
    client_name = getattr(module_to_generate, "__all__")

    report = create_empty_report()

    try:
        report["client"] = client_name
    except:
        report["client"] = []

    # Look for models first
    model_names = [model_name for model_name in module_to_generate.models.__all__ if model_name[0].isupper()]
    is_new_model = hasattr(module_to_generate, "_model_base")
    for model_name in model_names:
        model_cls = getattr(module_to_generate.models, model_name)
        if is_model(model_cls, is_new_model):
            report["models"]["models"][model_name] = create_model_report(model_cls, is_new_model)
        elif issubclass(model_cls, Exception):  # If not, might be an exception
            report["models"]["exceptions"][model_name] = create_model_report(model_cls, is_new_model)
        else:
            report["models"]["enums"][model_name] = create_model_report(model_cls, is_new_model)
    # Look for operation groups
    try:
        operations_classes = [op_name for op_name in module_to_generate.operations.__all__ if op_name[0].isupper()]
    except AttributeError:
        # This guy has no "operations", this is possible (Cognitive Services). Just skip it then.
        operations_classes = []

    for op_name in operations_classes:
        op_content = {"name": op_name}
        op_cls = getattr(module_to_generate.operations, op_name)
        for op_attr_name in dir(op_cls):
            op_attr = getattr(op_cls, op_attr_name)
            if isinstance(op_attr, types.FunctionType) and not op_attr_name.startswith("_"):
                # Keep it
                func_content = create_report_from_func(op_attr)
                op_content.setdefault("functions", {})[op_attr_name] = func_content
        report["operations"][op_name] = op_content

    return report


def get_attr_map(model_cls: object, is_new_model: bool) -> Dict[str, Any]:
    if is_new_model:
        return getattr(model_cls(), "_attr_to_rest_field")
    return getattr(model_cls, "_attribute_map")


def get_type_annotation(model_cls: object, attribute: str) -> List[str]:
    # make sure to get the annotations from the base class
    mros = model_cls.__mro__[:-3][::-1]
    annotations = {
        k: v
        for mro_class in mros
        if hasattr(mro_class, "__annotations__")  # pylint: disable=no-member
        for k, v in mro_class.__annotations__.items()  # pylint: disable=no-member
    }
    attr_type = annotations.get(attribute)
    type_list = getattr(attr_type, "__args__", [attr_type])
    return sorted(
        [
            (
                item.__forward_arg__.replace("_models.", "")
                if isinstance(item, ForwardRef)
                else getattr(item, "__name__", str(item))
            )
            for item in type_list
        ]
    )


def get_type(model_cls: object, attribute: str, conf: Dict[str, Any], is_new_model: bool) -> str:
    if is_new_model:
        return " or ".join(filter(lambda x: x != "NoneType", get_type_annotation(model_cls, attribute)))
    return conf["type"]


def _get_validation(model_cls: object, attribute: str) -> Dict[str, Any]:
    return getattr(model_cls, "_validation", {}).get(attribute, {})


def is_required(model_cls: object, attribute: str, is_new_model: bool) -> bool:
    if is_new_model:
        return "NoneType" not in get_type_annotation(model_cls, attribute)
    return _get_validation(model_cls, attribute).get("required", False)


def is_readonly(model_cls: object, attribute: str, is_new_model: bool) -> bool:
    if is_new_model:
        return getattr(getattr(model_cls, "_attr_to_rest_field").get(attribute), "_visibility") == ["read"]
    return _get_validation(model_cls, attribute).get("readonly", False)


def create_model_report(model_cls: object, is_new_model: bool):
    result = {
        "name": model_cls.__name__,
    }
    # If _attribute_map, it's a model
    if is_model(model_cls, is_new_model):
        result["type"] = "Model"
        for attribute, conf in get_attr_map(model_cls, is_new_model).items():
            result.setdefault("parameters", {})[attribute] = {
                "name": attribute,
                "properties": {
                    "type": get_type(model_cls, attribute, conf, is_new_model),
                    "required": is_required(model_cls, attribute, is_new_model),
                    "readonly": is_readonly(model_cls, attribute, is_new_model),
                },
            }
    elif issubclass(model_cls, Exception):  # If not, might be an exception
        result["type"] = "Exception"
    else:  # If not, it's an enum
        result["type"] = "Enum"
        result["values"] = list(model_cls.__members__)

    return result


def create_report_from_func(function_attr):
    func_content = {
        "name": function_attr.__name__,
        "metadata": getattr(function_attr, "metadata", {}),
        "parameters": [],
    }
    signature = inspect.signature(function_attr)
    for parameter_name in signature.parameters:
        if parameter_name == "self":
            continue
        if parameter_name == "custom_headers":
            break  # We reach Autorest generic
        parameter = signature.parameters[parameter_name]
        func_content["parameters"].append(
            {
                "name": parameter.name,
                "has_default_value": not (parameter.default is parameter.empty),
            }
        )
    return func_content


# given an input of a name, we need to return the appropriate relative diff between the sdk_root and the actual package directory
def resolve_package_directory(package_name):
    for install_file in ["setup.py", "pyproject.toml"]:
        packages = [
            os.path.dirname(p)
            for p in (
                glob.glob("{}/{}".format(package_name, install_file))
                + glob.glob("sdk/*/{}/{}".format(package_name, install_file))
            )
        ]
        if packages:
            break

    if len(packages) > 1:
        print(
            "There should only be a single package matched in either repository structure. The following were found: {}".format(
                packages
            )
        )
        sys.exit(1)

    return packages[0]


def merge_report(report_paths):
    """Merge report on the given paths list."""
    if len(report_paths) == 1:
        raise ValueError("Doesn't make sense to merge a report if there is only one report....")

    merged_report = create_empty_report()
    for report in sorted(report_paths):
        with open(report, "r") as report_fd:
            report_json = json.load(report_fd)

        merged_report["models"]["enums"].update(report_json["models"]["enums"])
        merged_report["models"]["exceptions"].update(report_json["models"]["exceptions"])
        merged_report["models"]["models"].update(report_json["models"]["models"])
        merged_report["operations"].update(report_json["operations"])
        merged_report["client"] = report_json["client"]
    return merged_report


# find last version or last stable version
def select_versions(versions: List[str], last_pypi_stable: bool) -> List[str]:
    versions.reverse()
    if last_pypi_stable:
        for version in versions:
            if not re.search("[a-zA-Z]", version):
                return [version]
        _LOGGER.info(f"Do not find stable version during {versions}")
        return [versions[0]]
    return [versions[0]]


def main(
    input_parameter: str,
    version: Optional[str] = None,
    no_venv: bool = False,
    pypi: bool = False,
    last_pypi: bool = False,
    output: Optional[str] = None,
    metadata_path: Optional[str] = None,
    last_pypi_stable: bool = False,
):
    output_msg = output if output else "default folder"
    _LOGGER.info(
        f"Building code report of {input_parameter} for version {version} in {output_msg} ({no_venv}/{pypi}/{last_pypi})"
    )
    package_name, module_name = parse_input(input_parameter)
    path_to_package = resolve_package_directory(package_name)

    output_filename = ""
    result = []
    if (version or pypi or last_pypi) and not no_venv:
        if version:
            versions = [version]
        else:
            _LOGGER.info(f"Download versions of {package_name} on PyPI")
            from pypi_tools.pypi import PyPIClient

            client = PyPIClient()
            versions = [str(v) for v in client.get_ordered_versions(package_name)]
            _LOGGER.info(f"Got {versions}")
            if last_pypi:
                _LOGGER.info(f"Only keep last PyPI version")
                versions = select_versions(versions, last_pypi_stable)

        for version in versions:
            _LOGGER.info(f"Installing version {version} of {package_name} in a venv")
            with create_venv_with_package(
                [f"{package_name}=={version}"]
            ) as venv, tempfile.TemporaryDirectory() as temp_dir:
                metadata_path = str(Path(temp_dir, f"metadata_{version}.json"))
                args = [
                    venv.env_exe,
                    __file__,
                    "--no-venv",
                    "--version",
                    version,
                    "--metadata",
                    metadata_path,
                    input_parameter,
                ]
                if output is not None:
                    args.append("--output=" + output)
                try:
                    subprocess.check_call(args)
                except subprocess.CalledProcessError:
                    # If it fail, just assume this version is too old to get an Autorest report
                    _LOGGER.warning(
                        f"Version {version} seems to be too old to build a report (probably not Autorest based)"
                    )
                # Files have been written by the subprocess
                with open(metadata_path, "r") as metadata_fd:
                    result.extend(json.load(metadata_fd)["reports_path"])
        # Files have been written by the subprocess
        return result

    modules = find_autorest_generated_folder(module_name)
    version = version or "latest"
    output_folder = Path(path_to_package) / Path("code_reports") / Path(version)
    output_folder.mkdir(parents=True, exist_ok=True)

    for module_name in modules:
        _LOGGER.info(f"Working on {module_name}")

        report = create_report(module_name)

        module_for_path = get_sub_module_part(package_name, module_name)
        if module_for_path:
            output_filename = output_folder / Path(module_for_path + ".json")
        else:
            if output is not None:
                output_filename = output
            else:
                output_filename = output_folder / Path("report.json")

        with open(output_filename, "w") as fd:
            json.dump(report, fd, indent=2)
            _LOGGER.info(f"Report written to {output_filename}")
        result.append(str(output_filename))

    if len(result) > 1:
        merged_report = merge_report(result)
        if output is not None:
            output_filename = output
        else:
            output_filename = output_folder / Path("merged_report.json")
        with open(output_filename, "w") as fd:
            json.dump(merged_report, fd, indent=2)
            _LOGGER.info(f"Merged report written to {output_filename}")
        result = [str(output_filename)]

    if metadata_path:
        metadata = {"reports_path": result}  # Prepare metadata
        with open(metadata_path, "w") as metadata_fd:
            _LOGGER.info(f"Writing metadata: {metadata}")
            json.dump(metadata, metadata_fd)

    return result


def find_autorest_generated_folder(module_prefix="azure"):
    """Find all Autorest generated code in that module prefix.
    This actually looks for a "models" package only (not file). We could be smarter if necessary.
    """
    _LOGGER.info(f"Looking for Autorest generated package in {module_prefix}")

    # Manually skip some namespaces for now
    if module_prefix in [
        "azure.cli",
        "azure.storage",
        "azure.servicemanagement",
        "azure.servicebus",
    ]:
        _LOGGER.info(f"Skip {module_prefix}")
        return []

    result = []
    try:
        _LOGGER.debug(f"Try {module_prefix}")
        model_module = importlib.import_module(".models", module_prefix)
        # If not exception, we MIGHT have found it, but cannot be a file.
        # Keep continue to try to break it, file module have no __path__
        model_module.__path__
        _LOGGER.info(f"Found {module_prefix}")
        result.append(module_prefix)
    except (ModuleNotFoundError, AttributeError):
        # No model, might dig deeper
        prefix_module = importlib.import_module(module_prefix)
        for _, sub_package, ispkg in pkgutil.iter_modules(prefix_module.__path__, module_prefix + "."):
            if ispkg:
                result += find_autorest_generated_folder(sub_package)
    return result


def get_sub_module_part(package_name, module_name):
    """Assuming package is azure-mgmt-compute and module name is azure.mgmt.compute.v2018-08-01
    will return v2018-08-01
    """
    sub_module_from_package = package_name.replace("-", ".")
    if not module_name.startswith(sub_module_from_package):
        _LOGGER.warning(f"Submodule {module_name} does not start with package name {package_name}")
        return
    return module_name[len(sub_module_from_package) + 1 :]


if __name__ == "__main__":
    import argparse

    parser = argparse.ArgumentParser(
        description="Code fingerprint building",
        formatter_class=argparse.RawTextHelpFormatter,
    )
    parser.add_argument("package_name", help="Package name.")
    parser.add_argument(
        "--version",
        "-v",
        dest="version",
        help="The version of the package you want. By default, latest and current branch.",
    )
    parser.add_argument(
        "--no-venv",
        dest="no_venv",
        action="store_true",
        help="If version is provided, this will assume the current accessible package is already this version. You should probably not use it.",
    )
    parser.add_argument(
        "--pypi",
        dest="pypi",
        action="store_true",
        help="If provided, build report for all versions on pypi of this package.",
    )
    parser.add_argument(
        "--last-pypi",
        dest="last_pypi",
        action="store_true",
        help="If provided, build report for last version on pypi of this package.",
    )
    parser.add_argument(
        "--last-pypi-stable",
        dest="last_pypi_stable",
        action="store_true",
        help="If provided, select last stable version on pypi",
    )
    parser.add_argument("--debug", dest="debug", action="store_true", help="Verbosity in DEBUG mode")
    parser.add_argument("--output", dest="output", help="Override output path.")
    parser.add_argument(
        "--metadata-path",
        dest="metadata",
        help="Write a metadata file about what happen. Mostly used for automation.",
    )
    args = parser.parse_args()

    logging.basicConfig(level=logging.DEBUG if args.debug else logging.INFO)

    main(
        args.package_name,
        args.version,
        args.no_venv,
        args.pypi,
        args.last_pypi,
        args.output,
        args.metadata,
    )



# pylint: disable=too-many-lines,line-too-long,useless-suppression
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
# cSpell:disable

from typing import Any, Dict, Optional
import pytest
import httpx
from devtools_testutils import recorded_by_proxy, RecordedTransport
from test_base import TestBase, servicePreparer
from openai import OpenAI
from azure.core.credentials import TokenCredential
from azure.ai.projects import AIProjectClient

BASE_OPENAI_UA = OpenAI(api_key="dummy").user_agent


class DummyTokenCredential(TokenCredential):
    def get_token(self, *scopes: str, **kwargs: Any):  # type: ignore[override]
        return None


@pytest.fixture
def patch_openai(monkeypatch):
    # Ensure no real network/token calls are made during the test.
    monkeypatch.setattr("azure.ai.projects._patch.get_bearer_token_provider", lambda *_, **__: "token-provider")


def _build_client(
    project_user_agent: Optional[str],
    default_headers: Optional[Dict[str, str]],
):
    project_client = AIProjectClient(
        "https://example.com/api/projects/test",
        DummyTokenCredential(),
        user_agent=project_user_agent,
    )
    kwargs: Dict[str, Any] = {"default_headers": default_headers}
    return project_client.get_openai_client(**kwargs)


class TestResponses(TestBase):

    # To run this test:
    # pytest tests\responses\test_responses.py::TestResponses::test_responses -s
    @servicePreparer()
    @recorded_by_proxy(RecordedTransport.HTTPX)
    def test_responses(self, **kwargs):
        """
        Test creating a responses call (no Agents, no Conversation).

        Routes used in this test:

        Action REST API Route                                OpenAI Client Method
        ------+---------------------------------------------+-----------------------------------
        POST   /openai/responses                             client.responses.create()
        """
        model = kwargs.get("foundry_model_name")

        client = self.create_client(operation_group="agents", **kwargs).get_openai_client()

        response1 = client.responses.create(
            model=model,
            input="How many feet in a mile?",
        )
        print(f"\nResponse id: {response1.id}, output text: {response1.output_text}")
        assert "5280" in response1.output_text or "5,280" in response1.output_text

        response2 = client.responses.create(
            model=model, input="And how many meters?", previous_response_id=response1.id
        )
        print(f"Response id: {response2.id}, output text: {response2.output_text}")
        assert "1609" in response2.output_text or "1,609" in response2.output_text

    @pytest.mark.parametrize(
        "project_ua,openai_default_header,expected_ua",
        [
            # 1) No user_agent anywhere
            (
                None,
                None,
                f"AIProjectClient {BASE_OPENAI_UA}",
            ),
            # 2) user_agent at project client only
            (
                "custom-client-ua",
                None,
                f"custom-client-ua-AIProjectClient {BASE_OPENAI_UA}",
            ),
            # 3) user_agent at openai client only
            (
                None,
                {"User-Agent": "custom-openai-ua"},
                "custom-openai-ua",
            ),
            # 4) user_agent at both clients only
            (
                "custom-client-ua",
                {"User-Agent": "custom-openai-ua"},
                "custom-openai-ua",
            ),
        ],
    )
    def test_user_agent_patching_via_response_create(
        self, project_ua, openai_default_header, expected_ua, patch_openai
    ):  # pylint: disable=redefined-outer-name,unused-argument
        client = _build_client(project_ua, openai_default_header)

        calls = []

        def fake_send(request: httpx.Request, *_args: Any, **kwargs: Any):
            # Capture headers that would be sent over the wire.
            calls.append(dict(request.headers))
            return httpx.Response(
                200,
                request=request,
                json={
                    "id": "resp_123",
                    "object": "response",
                    "model": kwargs.get("model", ""),
                    "status": "ok",
                    "output": "",
                },
            )

        # Monkeypatch the underlying httpx client used by the OpenAI client instance.
        client._client.send = fake_send  # type: ignore[attr-defined]

        # Act through the actual call surface
        client.responses.create(model="gpt-4o")

        # Assert
        assert calls, "Expected a responses.create call to be captured"
        headers_used = {k.lower(): v for k, v in calls[0].items()}

        assert headers_used["user-agent"] == expected_ua



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from .._utils.model_base import Model as _Model


class EmptyInput(_Model):
    """Empty model used in operation parameters."""


class EmptyInputOutput(_Model):
    """Empty model used in both parameter and return type."""


class EmptyOutput(_Model):
    """Empty model used in operation return type."""



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.appplatform.aio import AppPlatformManagementClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer
from devtools_testutils.aio import recorded_by_proxy_async

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestAppPlatformManagementConfigurationServicesOperationsAsync(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(AppPlatformManagementClient, is_async=True)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_get(self, resource_group):
        response = await self.client.configuration_services.get(
            resource_group_name=resource_group.name,
            service_name="str",
            configuration_service_name="str",
            api_version="2023-12-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_begin_create_or_update(self, resource_group):
        response = await (
            await self.client.configuration_services.begin_create_or_update(
                resource_group_name=resource_group.name,
                service_name="str",
                configuration_service_name="str",
                configuration_service_resource={
                    "id": "str",
                    "name": "str",
                    "properties": {
                        "generation": "Gen1",
                        "instances": [{"name": "str", "status": "str"}],
                        "provisioningState": "str",
                        "resourceRequests": {"cpu": "str", "instanceCount": 0, "memory": "str"},
                        "settings": {
                            "gitProperty": {
                                "repositories": [
                                    {
                                        "label": "str",
                                        "name": "str",
                                        "patterns": ["str"],
                                        "uri": "str",
                                        "caCertResourceId": "str",
                                        "gitImplementation": "str",
                                        "hostKey": "str",
                                        "hostKeyAlgorithm": "str",
                                        "password": "str",
                                        "privateKey": "str",
                                        "searchPaths": ["str"],
                                        "strictHostKeyChecking": bool,
                                        "username": "str",
                                    }
                                ]
                            }
                        },
                    },
                    "systemData": {
                        "createdAt": "2020-02-20 00:00:00",
                        "createdBy": "str",
                        "createdByType": "str",
                        "lastModifiedAt": "2020-02-20 00:00:00",
                        "lastModifiedBy": "str",
                        "lastModifiedByType": "str",
                    },
                    "type": "str",
                },
                api_version="2023-12-01",
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_begin_delete(self, resource_group):
        response = await (
            await self.client.configuration_services.begin_delete(
                resource_group_name=resource_group.name,
                service_name="str",
                configuration_service_name="str",
                api_version="2023-12-01",
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_list(self, resource_group):
        response = self.client.configuration_services.list(
            resource_group_name=resource_group.name,
            service_name="str",
            api_version="2023-12-01",
        )
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_begin_validate(self, resource_group):
        response = await (
            await self.client.configuration_services.begin_validate(
                resource_group_name=resource_group.name,
                service_name="str",
                configuration_service_name="str",
                settings={
                    "gitProperty": {
                        "repositories": [
                            {
                                "label": "str",
                                "name": "str",
                                "patterns": ["str"],
                                "uri": "str",
                                "caCertResourceId": "str",
                                "gitImplementation": "str",
                                "hostKey": "str",
                                "hostKeyAlgorithm": "str",
                                "password": "str",
                                "privateKey": "str",
                                "searchPaths": ["str"],
                                "strictHostKeyChecking": bool,
                                "username": "str",
                            }
                        ]
                    }
                },
                api_version="2023-12-01",
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_begin_validate_resource(self, resource_group):
        response = await (
            await self.client.configuration_services.begin_validate_resource(
                resource_group_name=resource_group.name,
                service_name="str",
                configuration_service_name="str",
                configuration_service_resource={
                    "id": "str",
                    "name": "str",
                    "properties": {
                        "generation": "Gen1",
                        "instances": [{"name": "str", "status": "str"}],
                        "provisioningState": "str",
                        "resourceRequests": {"cpu": "str", "instanceCount": 0, "memory": "str"},
                        "settings": {
                            "gitProperty": {
                                "repositories": [
                                    {
                                        "label": "str",
                                        "name": "str",
                                        "patterns": ["str"],
                                        "uri": "str",
                                        "caCertResourceId": "str",
                                        "gitImplementation": "str",
                                        "hostKey": "str",
                                        "hostKeyAlgorithm": "str",
                                        "password": "str",
                                        "privateKey": "str",
                                        "searchPaths": ["str"],
                                        "strictHostKeyChecking": bool,
                                        "username": "str",
                                    }
                                ]
                            }
                        },
                    },
                    "systemData": {
                        "createdAt": "2020-02-20 00:00:00",
                        "createdBy": "str",
                        "createdByType": "str",
                        "lastModifiedAt": "2020-02-20 00:00:00",
                        "lastModifiedBy": "str",
                        "lastModifiedByType": "str",
                    },
                    "type": "str",
                },
                api_version="2023-12-01",
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
from collections.abc import MutableMapping
from typing import Any, Callable, Optional, TypeVar

from azure.core import AsyncPipelineClient
from azure.core.async_paging import AsyncItemPaged, AsyncList
from azure.core.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceExistsError,
    ResourceNotFoundError,
    ResourceNotModifiedError,
    map_error,
)
from azure.core.pipeline import PipelineResponse
from azure.core.rest import AsyncHttpResponse, HttpRequest
from azure.core.tracing.decorator import distributed_trace

from ..... import models as _models4
from ....._utils.model_base import _deserialize
from ....._utils.serialization import Deserializer, Serializer
from .....aio._configuration import PageableClientConfiguration
from ...operations._operations import (
    build_server_driven_pagination_continuation_token_request_header_nested_response_body_request,
    build_server_driven_pagination_continuation_token_request_header_response_body_request,
    build_server_driven_pagination_continuation_token_request_header_response_header_request,
    build_server_driven_pagination_continuation_token_request_query_nested_response_body_request,
    build_server_driven_pagination_continuation_token_request_query_response_body_request,
    build_server_driven_pagination_continuation_token_request_query_response_header_request,
)

T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, AsyncHttpResponse], T, dict[str, Any]], Any]]


class ServerDrivenPaginationContinuationTokenOperations:  # pylint: disable=name-too-long
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~payload.pageable.aio.PageableClient`'s
        :attr:`continuation_token` attribute.
    """

    def __init__(self, *args, **kwargs) -> None:
        input_args = list(args)
        self._client: AsyncPipelineClient = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config: PageableClientConfiguration = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize: Serializer = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize: Deserializer = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @distributed_trace
    def request_query_response_body(
        self, *, foo: Optional[str] = None, bar: Optional[str] = None, **kwargs: Any
    ) -> AsyncItemPaged["_models4.Pet"]:
        """request_query_response_body.

        :keyword foo: Default value is None.
        :paramtype foo: str
        :keyword bar: Default value is None.
        :paramtype bar: str
        :return: An iterator like instance of Pet
        :rtype: ~azure.core.async_paging.AsyncItemPaged[~payload.pageable.models.Pet]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[list[_models4.Pet]] = kwargs.pop("cls", None)

        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        def prepare_request(_continuation_token=None):

            _request = build_server_driven_pagination_continuation_token_request_query_response_body_request(
                token=_continuation_token,
                foo=foo,
                bar=bar,
                headers=_headers,
                params=_params,
            )
            path_format_arguments = {
                "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
            }
            _request.url = self._client.format_url(_request.url, **path_format_arguments)
            return _request

        async def extract_data(pipeline_response):
            deserialized = pipeline_response.http_response.json()
            list_of_elem = _deserialize(
                list[_models4.Pet],
                deserialized.get("pets", []),
            )
            if cls:
                list_of_elem = cls(list_of_elem)  # type: ignore
            return deserialized.get("nextToken") or None, AsyncList(list_of_elem)

        async def get_next(_continuation_token=None):
            _request = prepare_request(_continuation_token)

            _stream = False
            pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
                _request, stream=_stream, **kwargs
            )
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                raise HttpResponseError(response=response)

            return pipeline_response

        return AsyncItemPaged(get_next, extract_data)

    @distributed_trace
    def request_header_response_body(
        self, *, foo: Optional[str] = None, bar: Optional[str] = None, **kwargs: Any
    ) -> AsyncItemPaged["_models4.Pet"]:
        """request_header_response_body.

        :keyword foo: Default value is None.
        :paramtype foo: str
        :keyword bar: Default value is None.
        :paramtype bar: str
        :return: An iterator like instance of Pet
        :rtype: ~azure.core.async_paging.AsyncItemPaged[~payload.pageable.models.Pet]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[list[_models4.Pet]] = kwargs.pop("cls", None)

        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        def prepare_request(_continuation_token=None):

            _request = build_server_driven_pagination_continuation_token_request_header_response_body_request(
                token=_continuation_token,
                foo=foo,
                bar=bar,
                headers=_headers,
                params=_params,
            )
            path_format_arguments = {
                "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
            }
            _request.url = self._client.format_url(_request.url, **path_format_arguments)
            return _request

        async def extract_data(pipeline_response):
            deserialized = pipeline_response.http_response.json()
            list_of_elem = _deserialize(
                list[_models4.Pet],
                deserialized.get("pets", []),
            )
            if cls:
                list_of_elem = cls(list_of_elem)  # type: ignore
            return deserialized.get("nextToken") or None, AsyncList(list_of_elem)

        async def get_next(_continuation_token=None):
            _request = prepare_request(_continuation_token)

            _stream = False
            pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
                _request, stream=_stream, **kwargs
            )
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                raise HttpResponseError(response=response)

            return pipeline_response

        return AsyncItemPaged(get_next, extract_data)

    @distributed_trace
    def request_query_response_header(
        self, *, foo: Optional[str] = None, bar: Optional[str] = None, **kwargs: Any
    ) -> AsyncItemPaged["_models4.Pet"]:
        """request_query_response_header.

        :keyword foo: Default value is None.
        :paramtype foo: str
        :keyword bar: Default value is None.
        :paramtype bar: str
        :return: An iterator like instance of Pet
        :rtype: ~azure.core.async_paging.AsyncItemPaged[~payload.pageable.models.Pet]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[list[_models4.Pet]] = kwargs.pop("cls", None)

        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        def prepare_request(_continuation_token=None):

            _request = build_server_driven_pagination_continuation_token_request_query_response_header_request(
                token=_continuation_token,
                foo=foo,
                bar=bar,
                headers=_headers,
                params=_params,
            )
            path_format_arguments = {
                "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
            }
            _request.url = self._client.format_url(_request.url, **path_format_arguments)
            return _request

        async def extract_data(pipeline_response):
            deserialized = pipeline_response.http_response.json()
            list_of_elem = _deserialize(
                list[_models4.Pet],
                deserialized.get("pets", []),
            )
            if cls:
                list_of_elem = cls(list_of_elem)  # type: ignore
            return pipeline_response.http_response.headers.get("next-token") or None, AsyncList(list_of_elem)

        async def get_next(_continuation_token=None):
            _request = prepare_request(_continuation_token)

            _stream = False
            pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
                _request, stream=_stream, **kwargs
            )
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                raise HttpResponseError(response=response)

            return pipeline_response

        return AsyncItemPaged(get_next, extract_data)

    @distributed_trace
    def request_header_response_header(
        self, *, foo: Optional[str] = None, bar: Optional[str] = None, **kwargs: Any
    ) -> AsyncItemPaged["_models4.Pet"]:
        """request_header_response_header.

        :keyword foo: Default value is None.
        :paramtype foo: str
        :keyword bar: Default value is None.
        :paramtype bar: str
        :return: An iterator like instance of Pet
        :rtype: ~azure.core.async_paging.AsyncItemPaged[~payload.pageable.models.Pet]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[list[_models4.Pet]] = kwargs.pop("cls", None)

        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        def prepare_request(_continuation_token=None):

            _request = build_server_driven_pagination_continuation_token_request_header_response_header_request(
                token=_continuation_token,
                foo=foo,
                bar=bar,
                headers=_headers,
                params=_params,
            )
            path_format_arguments = {
                "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
            }
            _request.url = self._client.format_url(_request.url, **path_format_arguments)
            return _request

        async def extract_data(pipeline_response):
            deserialized = pipeline_response.http_response.json()
            list_of_elem = _deserialize(
                list[_models4.Pet],
                deserialized.get("pets", []),
            )
            if cls:
                list_of_elem = cls(list_of_elem)  # type: ignore
            return pipeline_response.http_response.headers.get("next-token") or None, AsyncList(list_of_elem)

        async def get_next(_continuation_token=None):
            _request = prepare_request(_continuation_token)

            _stream = False
            pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
                _request, stream=_stream, **kwargs
            )
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                raise HttpResponseError(response=response)

            return pipeline_response

        return AsyncItemPaged(get_next, extract_data)

    @distributed_trace
    def request_query_nested_response_body(
        self, *, foo: Optional[str] = None, bar: Optional[str] = None, **kwargs: Any
    ) -> AsyncItemPaged["_models4.Pet"]:
        """request_query_nested_response_body.

        :keyword foo: Default value is None.
        :paramtype foo: str
        :keyword bar: Default value is None.
        :paramtype bar: str
        :return: An iterator like instance of Pet
        :rtype: ~azure.core.async_paging.AsyncItemPaged[~payload.pageable.models.Pet]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[list[_models4.Pet]] = kwargs.pop("cls", None)

        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        def prepare_request(_continuation_token=None):

            _request = build_server_driven_pagination_continuation_token_request_query_nested_response_body_request(
                token=_continuation_token,
                foo=foo,
                bar=bar,
                headers=_headers,
                params=_params,
            )
            path_format_arguments = {
                "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
            }
            _request.url = self._client.format_url(_request.url, **path_format_arguments)
            return _request

        async def extract_data(pipeline_response):
            deserialized = pipeline_response.http_response.json()
            list_of_elem = _deserialize(
                list[_models4.Pet],
                deserialized.get("nestedItems", {}).get("pets", []),
            )
            if cls:
                list_of_elem = cls(list_of_elem)  # type: ignore
            return deserialized.get("nestedNext", {}).get("nextToken") or None, AsyncList(list_of_elem)

        async def get_next(_continuation_token=None):
            _request = prepare_request(_continuation_token)

            _stream = False
            pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
                _request, stream=_stream, **kwargs
            )
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                raise HttpResponseError(response=response)

            return pipeline_response

        return AsyncItemPaged(get_next, extract_data)

    @distributed_trace
    def request_header_nested_response_body(
        self, *, foo: Optional[str] = None, bar: Optional[str] = None, **kwargs: Any
    ) -> AsyncItemPaged["_models4.Pet"]:
        """request_header_nested_response_body.

        :keyword foo: Default value is None.
        :paramtype foo: str
        :keyword bar: Default value is None.
        :paramtype bar: str
        :return: An iterator like instance of Pet
        :rtype: ~azure.core.async_paging.AsyncItemPaged[~payload.pageable.models.Pet]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[list[_models4.Pet]] = kwargs.pop("cls", None)

        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        def prepare_request(_continuation_token=None):

            _request = build_server_driven_pagination_continuation_token_request_header_nested_response_body_request(
                token=_continuation_token,
                foo=foo,
                bar=bar,
                headers=_headers,
                params=_params,
            )
            path_format_arguments = {
                "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
            }
            _request.url = self._client.format_url(_request.url, **path_format_arguments)
            return _request

        async def extract_data(pipeline_response):
            deserialized = pipeline_response.http_response.json()
            list_of_elem = _deserialize(
                list[_models4.Pet],
                deserialized.get("nestedItems", {}).get("pets", []),
            )
            if cls:
                list_of_elem = cls(list_of_elem)  # type: ignore
            return deserialized.get("nestedNext", {}).get("nextToken") or None, AsyncList(list_of_elem)

        async def get_next(_continuation_token=None):
            _request = prepare_request(_continuation_token)

            _stream = False
            pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
                _request, stream=_stream, **kwargs
            )
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                raise HttpResponseError(response=response)

            return pipeline_response

        return AsyncItemPaged(get_next, extract_data)



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
from devtools_testutils import AzureRecordedTestCase
from specs.azure.clientgenerator.core.clientinitialization.individually.aio import (
    IndividuallyNestedWithHeaderClient,
    IndividuallyNestedWithMixedClient,
    IndividuallyNestedWithMultipleClient,
    IndividuallyNestedWithParamAliasClient,
    IndividuallyNestedWithPathClient,
    IndividuallyNestedWithQueryClient,
)


class IndividuallyNestedWithPathClientTestBaseAsync(AzureRecordedTestCase):

    def create_async_client(self, endpoint):
        credential = self.get_credential(IndividuallyNestedWithPathClient, is_async=True)
        return self.create_client_from_credential(
            IndividuallyNestedWithPathClient,
            credential=credential,
            endpoint=endpoint,
        )


class IndividuallyNestedWithQueryClientTestBaseAsync(AzureRecordedTestCase):

    def create_async_client(self, endpoint):
        credential = self.get_credential(IndividuallyNestedWithQueryClient, is_async=True)
        return self.create_client_from_credential(
            IndividuallyNestedWithQueryClient,
            credential=credential,
            endpoint=endpoint,
        )


class IndividuallyNestedWithHeaderClientTestBaseAsync(AzureRecordedTestCase):

    def create_async_client(self, endpoint):
        credential = self.get_credential(IndividuallyNestedWithHeaderClient, is_async=True)
        return self.create_client_from_credential(
            IndividuallyNestedWithHeaderClient,
            credential=credential,
            endpoint=endpoint,
        )


class IndividuallyNestedWithMultipleClientTestBaseAsync(AzureRecordedTestCase):

    def create_async_client(self, endpoint):
        credential = self.get_credential(IndividuallyNestedWithMultipleClient, is_async=True)
        return self.create_client_from_credential(
            IndividuallyNestedWithMultipleClient,
            credential=credential,
            endpoint=endpoint,
        )


class IndividuallyNestedWithMixedClientTestBaseAsync(AzureRecordedTestCase):

    def create_async_client(self, endpoint):
        credential = self.get_credential(IndividuallyNestedWithMixedClient, is_async=True)
        return self.create_client_from_credential(
            IndividuallyNestedWithMixedClient,
            credential=credential,
            endpoint=endpoint,
        )


class IndividuallyNestedWithParamAliasClientTestBaseAsync(AzureRecordedTestCase):

    def create_async_client(self, endpoint):
        credential = self.get_credential(IndividuallyNestedWithParamAliasClient, is_async=True)
        return self.create_client_from_credential(
            IndividuallyNestedWithParamAliasClient,
            credential=credential,
            endpoint=endpoint,
        )
